PV.import("CCMSupport.js");
PV.import("CCMConfig.js");




function logMessage(type, logdata) {
   if ( DEBUG ) {
       PV.Logger.logInfo("DEBUG", type, logdata);
   }
}
 

logMessage("CCMDEBUG", "Beginning CCM Adaptor...");


var modelInterface = PV.ModelInterface();

logMessage("CCMDEBUG", "Setting modelInterface...");

PV.adaptor.modelInterface = modelInterface;

logMessage("CCMDEBUG", "Running PV.adaptor.modelInterface.initializeForInventory");

PV.adaptor.modelInterface.initializeForInventory();


logMessage("CCMDEBUG", "Running modelInterface.Subelement().prototype.addFloatMetric");

modelInterface.Subelement().prototype.addFloatMetric = 
function addFloatMetric(path,name,value,timestamp) {
  var metric;
  if (value == undefined) return;
  if (isNaN(value)) return;
  metric = modelInterface.FloatMetric();
  if (path) {
    var pname = path + name;
    metric.name = pname.toString();
  } else {
    metric.name = name.toString();
  }
  metric.value = value;
  if (timestamp) {
    metric.timestamp = timestamp;
  }
  return this.addMetric(metric);
};

modelInterface.Subelement().prototype.addAggregatingFloatMetric = 
  function addAggregatingFloatMetric(path,name,value,type,interval,timestamp) {
  var metric;
  if (value == undefined) return;
  if (isNaN(value)) return;
  metric = modelInterface.FloatMetric()
  if (path) {
    var pname = path + name;
    metric.name = pname.toString();
  } else {
    metric.name = name.toString();
  }
  metric.value = value;
  if (timestamp) {
    metric.timestamp = timestamp;
  }
  metric.aggregation = type;
  metric.aggregationInterval = interval;
  return this.addAggregatingMetric(metric);
};

PV.adaptor.initializeSchemas = 
function initializeSchemas() {
  logMessage("CCMDEBUG", "Initializing schema...");
  var CDRCMRSchema = 
  PV.SQLRecordSchema(
		     'CDRCMR',
		     {   onRead: processCDR,
                         maxOccurs: Infinity,
			 query: query
			 },
		     PV.StringFieldSchema('globalCallID_callManagerId'),
		     PV.StringFieldSchema('globalCallID_CallId'),
		     PV.StringFieldSchema('origLegCallIdentifier'),
		     PV.StringFieldSchema('destLegIdentifier'),
		     PV.StringFieldSchema('callIdentifier'),
		     PV.NumberFieldSchema('dateTimeOrigination'),
		     PV.NumberFieldSchema('dateTimeConnect'),
		     PV.NumberFieldSchema('dateTimeDisconnect'),
		     PV.NumberFieldSchema('duration'),
		     PV.NumberFieldSchema('origIpAddress',
		       {
			 constructor: function (string) {
			   return (new Number(string).toUInt32())
			 },
			 externalName: 'origIpAddr'
		       }),
		     PV.NumberFieldSchema('destIpAddress',
		       {
			 constructor: function (string) {
			   return (new Number(string).toUInt32())
			 },
			 externalName: 'destIpAddr'
		       }),
		     PV.StringFieldSchema('callingPartyNumber',
		       {
			 constructor: function (string) {
			   return (new Extension(string))
			 }
		       }),
		     PV.StringFieldSchema('originalCalledPartyNumber',
		       {
			 constructor: function (string) {
			   return (new Extension(string))
			 }
		       }),
		     PV.StringFieldSchema('finalCalledPartyNumber',
		       {
			 constructor: function (string) {
			   return (new Extension(string))
			 }
		       }),
		     PV.StringFieldSchema('callingPartyNumberPartition'),
		     PV.StringFieldSchema('originalCalledPartyNumberPartition'),
		     PV.StringFieldSchema('finalCalledPartyNumberPartition'),
		     PV.StringFieldSchema('origDeviceName'),
		     PV.StringFieldSchema('destDeviceName'),
		     PV.StringFieldSchema('origCause_value',
		       {
			 constructor: function (string) {
			   return (new CauseCode(string))
			 }
		       }),
		     PV.NumberFieldSchema('destCause_value',
		       {
			 constructor: function (string) {
			   return (new CauseCode(string))
			 }
		       }),
		     PV.NumberFieldSchema('numberPacketsSent'),
		     PV.NumberFieldSchema('numberOctetsSent'),
		     PV.NumberFieldSchema('numberPacketsReceived'),
		     PV.NumberFieldSchema('numberOctetsReceived'),
		     PV.NumberFieldSchema('numberPacketsLost'),
		     PV.NumberFieldSchema('jitter'),
		     PV.NumberFieldSchema('latency'));
  
  this.schemas.add(
		   PV.SQLInputSchema('CCMInput',
		   {onRead: persistSave},
				     CDRCMRSchema
				     ));
  defineCDRClass(CDRCMRSchema.recordPrototype);
};

function processCDR(record) {
  logMessage("CCMDEBUG", "In processCDR()");
  handleCDRCMR(record);
  processNonAggregated(record);
  processAggregated(record);
}

function padZeros(subelement) {
   var codes = defineCauseCodeMetricNames();
   for ( var code in codes ) {
       var c = new CauseCode(code);
       subelement.addAggregatingFloatMetric(c.metricName(),0,'sum',300);
   }
}

function processNonAggregated(record) {
        logMessage("CCMDEBUG", "In processNonAggregated()");
	var element, subelement, metric;
	element = modelInterface.Element();
	element.timestamp = record.dateTimeOrigination;
	element.name = CMCLUSTER;
	if (record.callingPartyNumber.isSimpleExtension()) {
	  handleCallingPartySimpleExtension(element,record);
	}
	if (record.originalCalledPartyNumber.isSimpleExtension()) {
	  handleOriginalCalledPartySimpleExtension(element,record);
	}
	if (record.isSimpleExtCallForward()) {
	  handleSimpleExtCallForward(element,record);
	}
	modelInterface.commit();
}

function processAggregated(record) {
        logMessage("CCMDEBUG", "In processAggregated()");
	var element, subelement, metric;
	element = modelInterface.Element();
	element.timestamp = record.dateTimeOrigination;
	element.name = CMCLUSTER;
	subelement = modelInterface.Subelement();
	subelement.name = record.nullSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Cluster';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterTotalCall',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterSuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterUnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterCallDuration',record.duration,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterInternalCall',record.isInternalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterOnNetCall',record.isOnNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterTollBypassCall',record.isTollBypassCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIncomingCall',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterLocalCall',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterInternationalCall',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterLongDistanceCall',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterVoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	if (record.hasPacketReceived()) {
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterNbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterNbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterNbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterNbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterNbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterJitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterLatency',record.getLatency(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterMOS',record.meanOpinionScore(),'average',300);
	}
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterDropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterOffNetCall',record.isOffNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterOutboundExternalCallDuration',record.outboundExternalCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterOutboundExternalSuccessfulCall',record.isOutboundExternalSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterIncomingSuccessfulCall',record.isIncomingSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterVoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterVoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CLUSTER_PATH,'ClusterTollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(record.origCause_value.metricNameForCluster(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
	if (record.callingPartyNumber.isAggregatedExtension()) {
	  handleCallingPartyAggregatedExtension(element,record);
	}
	if (record.originalCalledPartyNumber.isAggregatedExtension()) {
	  handleOriginalCalledPartyAggregatedExtension(element,record);
	}
	if (record.isAggregatedExtCallForward()) {
	  handleAggregatedExtCallForward(element,record);
	}
	if (record.isGateway()) {
	  handleGateway(element,record);
	}
	if (record.isPath()) {
	  handlePath(element,record);
	}
	modelInterface.commit();
}

function handleCallingPartySimpleExtension(element,record) {
        logMessage("CCMDEBUG", "In handleCallingPartySimpleExtension()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.originatingSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('siteId',record.callingPartyNumber.getSiteName());
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);
	subelement.addFloatMetric(FRML_CDR_PATH,'isCaller',1);
	subelement.addFloatMetric(FRML_CDR_PATH,'isCallee',0);
	subelement.addFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration);
	subelement.addFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'TollBypassCall',record.isTollBypassCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN());
	subelement.addFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OutboundSuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets());
	subelement.addFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration());
	if (record.hasOriginDetails()) {
		subelement.addFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore());
	}
	if (record.hasOriginDetails()) {
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost());
		subelement.addFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter());
		subelement.addFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency());
	}
	subelement.addFloatMetric(record.origCause_value.metricName(),1);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handleOriginalCalledPartySimpleExtension(element,record) {
        logMessage("CCMDEBUG", "In handleOriginalCalledPartySimpleExtension()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.destinationSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('siteId',record.originalCalledPartyNumber.getSiteName());
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);

	subelement.addFloatMetric(FRML_CDR_PATH,'isCaller',0);
	subelement.addFloatMetric(FRML_CDR_PATH,'isCallee',1);
	subelement.addFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'CallDuration',record.destinationDuration());
	subelement.addFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN());
	subelement.addFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'InboundSuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets());
	subelement.addFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration());
	if (record.hasDestinationDetails()) {
		subelement.addFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore());
	}
	if (record.hasDestinationDetails()) {
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost());
		subelement.addFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter());
		subelement.addFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency());
	}
	subelement.addFloatMetric(record.origCause_value.metricName(),1);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handleSimpleExtCallForward(element,record) {
        logMessage("CCMDEBUG", "In handleSimpleExtCallForward()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.forwardedSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('siteId',record.finalCalledPartyNumber.getSiteName());
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);

	subelement.addFloatMetric(FRML_CDR_PATH,'isCaller',0);
	subelement.addFloatMetric(FRML_CDR_PATH,'isCallee',1);
	subelement.addFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration);
	subelement.addFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension());
	subelement.addFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN());
	subelement.addFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'InboundSuccessfulCall',record.isSuccessfulCall());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration());
	subelement.addFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets());
	subelement.addFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration());
	if (record.hasDestinationDetails()) {
		subelement.addFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore());
	}
	if (record.hasDestinationDetails()) {
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived());
		subelement.addFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost());
		subelement.addFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter());
		subelement.addFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency());
	}
	subelement.addFloatMetric(record.origCause_value.metricName(),1);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handleCallingPartyAggregatedExtension(element,record) {
        logMessage("CCMDEBUG", "In handleCallingPartyAggregatedExtension()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.originatingSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);

	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCaller',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCallee',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollBypassCall',record.isTollBypassCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OutboundSuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	if (record.hasOriginDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore(),'average',300);
	}
	if (record.hasOriginDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency(),'average',300);
	}
	subelement.addAggregatingFloatMetric(record.origCause_value.metricName(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}


function handleOriginalCalledPartyAggregatedExtension(element,record) {
        logMessage("CCMDEBUG", "In handleOriginalCalledPartyAggregatedExtension()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.destinationSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);

	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCaller',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCallee',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'CallDuration',record.destinationDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'InboundSuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	if (record.hasDestinationDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore(),'average',300);
	}
	if (record.hasDestinationDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency(),'average',300);
	}
	subelement.addAggregatingFloatMetric(record.origCause_value.metricName(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handleAggregatedExtCallForward(element,record) {
        logMessage("CCMDEBUG", "In handleAggregatedExtCallForward()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.forwardedSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Extension';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('cdrType','Extension');
	element.addSubelement(subelement);

	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCaller',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCallee',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'InboundSuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	if (record.hasDestinationDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore(),'average',300);
	}
	if (record.hasDestinationDetails()) {
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency(),'average',300);
	}
	subelement.addAggregatingFloatMetric(record.origCause_value.metricName(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handleGateway(element,record) {
        logMessage("CCMDEBUG", "In handleGateway()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.gatewaySubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Gateway';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('siteId',record.getSiteNameFromPSTNGateway());
	subelement.addProperty('cdrType','Gateway');
	element.addSubelement(subelement);

	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCaller',record.isGatewayCaller(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCallee',record.isGatewayCallee(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Internal',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OnNet',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollBypassCall',record.isTollBypassCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OffNet',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OutboundSuccessfulCall',record.isOutboundExternalSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'InboundSuccessfulCall',record.isIncomingSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	if (record.hasPacketReceived()) {
		subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore(),'average',300);
	}
	if (record.hasPacketReceived()) {
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency(),'average',300);
	}
	subelement.addAggregatingFloatMetric(record.origCause_value.metricName(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}

function handlePath(element,record) {
        logMessage("CCMDEBUG", "In handlePath()");
	var subelement = modelInterface.Subelement();
	subelement.name = record.pathSubelementName(element);
	subelement.family = 'VOIP_Cisco_CDR_Path';
	subelement.addProperty('clusterId',CMCLUSTER);
	subelement.addProperty('customerId',CUSTOMERNAME);
	subelement.addProperty('cdrType','Path');
	element.addSubelement(subelement);

	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCaller',1,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'isCallee',0,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'SuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'UnsuccessfulCall',record.isUnsuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ShortCall',record.isShortCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'CallDuration',record.duration,'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Internal',record.isInternalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OnNet',record.isOnNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollBypassCall',record.isTollBypassCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteIPCall',record.isIntrasiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteIPCall',record.isIntersiteIPCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntrasiteLegacyPBXCall',record.isIntrasiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'IntersiteLegacyPBXCall',record.isIntersiteLegacyPBXCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Incoming',record.isIncomingCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'Local',record.isLocalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'International',record.isInternationalCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'LongDistance',record.isLongDistanceCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoiceMailCall',record.isVoiceMailCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToVoiceMail',record.isCallForwardToVoiceMail(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToExtension',record.isCallForwardToExtension(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'ForwardToPSTN',record.isCallForwardToPSTN(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'DropCall',record.isDropCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OffNet',record.isOffNetCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'OutboundSuccessfulCall',record.isSuccessfulCall(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailCallDuration',record.voicemailCallDuration(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'VoicemailPackets',record.voicemailPackets(),'sum',300);
	subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'TollbypassCallDuration',record.tollbypassCallDuration(),'sum',300);
	if (record.hasPacketReceived()) {
		subelement.addAggregatingFloatMetric(FRML_CDR_PATH,'MOS',record.meanOpinionScore(),'average',300);
	}
	if (record.hasPacketReceived()) {
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsSent',record.getPacketsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsSent',record.getOctetsSent(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsReceived',record.getPacketsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbOctetsReceived',record.getOctetsReceived(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'NbPacketsLost',record.getPacketsLost(),'sum',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Jitter',record.getJitter(),'average',300);
		subelement.addAggregatingFloatMetric(FRML_CMR_PATH,'Latency',record.getLatency(),'average',300);
	}
	subelement.addAggregatingFloatMetric(record.origCause_value.metricName(),1,'sum',300);
        if ( PAD_ZERO == "true" ) {
           padZeros(subelement);
        }
}
