//
// Jean-Francois Catz - October, 16th 2003
//
// 11/07/2003 - Add Cluster subelement support		    		(Jfc)
// 11/10/2003 - Decode PSTN Gateways IP addresses	    		(Jfc)
// 11/25/2003 - Add PSTN Gateway subelement support	   		(Jfc)
//	      - Add siteId Extension support		   		(Jfc)
//	      - Add PATH subelement support				(Jfc)
// 11/28/2003 - Add Multiple Countries offices per Cluster		(Jfc)
// 12/12/2003 - Modify isInternalExtension()				(Jfc)
//	      - Add Intrasite/Intersite/IP/LegacyPBX Call		(Jfc)
// 01/29/2004 - Add functions for 5mn UBA aggregation			(Jfc)
// 03/09/2004 - Add Configuration Helpers + code optimization		(Dpen)
// 03/16/2004 - Add getPacketsLost function				(Jfc)
// 28/07/2004 - Code optimization (Extension methods) - new UBA release	(Dpen)
// 23/03/2006 - Update to V4.4 version of procedural API                (DPen)
//
// (C) Quallaby, Corp.
//


//
//
// Formula Paths
//



var FRML_CDR_PATH ="AP~Specific~Bulk~VOIP~Cisco~CDR~";
var FRML_CMR_PATH ="AP~Specific~Bulk~VOIP~Cisco~CMR~";
var FRML_CLUSTER_PATH ="AP~Specific~Bulk~VOIP~Cisco~CLUSTER~";


//
// Init variables for LAST_TIME and Max number of result set
//

var QUERY_LIMIT = 5000; 			// Change value to 10000 for real deployment (5000 is for test only)
var Persist = PV.PersistentObject('LAST_TIME.xml');

function initCDRStartTime(startTime) {
	if (!Persist.LAST_TIME) {
  		Persist.LAST_TIME = startTime;
 		Persist.save();
	}
}

function persistSave() {
	PV.Logger.logInfo("persistSave",1,Persist.LAST_TIME);
	Persist.save();
}

//
// CISCO IP ADDRESS DECODING SECTION
//

function convertInt2IPAddress(ip) {
   var hexa = Number(ip).toString(16);
   var hexb = '0x'.concat(hexa.substring(6,8),
                                hexa.substring(4,6),
                                hexa.substring(2,4),
                                hexa.substring(0,2));
  if (hexa.length < 8) {
    hexa = "0"+hexa;
  }


//   return Number(hexb);

  var one = hexa.substring(0, 2);
  one = Number(one);
  var two = hexa.substring(2, 4);
  two = Number(two);
  var three = hexa.substring(4, 6);
  three = Number(three);
  var four = hexa.substring(6, 8);
  four = Number(four);

  var resip = four + "." + three + "." + two + "." + one;
  return resip;
} 

function convertIPAddress2Number(ip) {
  var fields = ip.split(".");
  var hex3 = Number(fields[3]).toString(16); 
  var hex2 = Number(fields[2]).toString(16); 
  var hex1 = Number(fields[1]).toString(16); 
  var hex0 = Number(fields[0]).toString(16); 

  if (hex0.length < 2) {
    hex0 = '0' + hex0;
  }

  if (hex1.length < 2) {
    hex1 = '0' + hex1;
  }

  if (hex2.length < 2) {
    hex2 = '0' + hex2;
  }

  if (hex3.length < 2) {
    hex3 = '0' + hex3;
  }

  var hexa = '0x'.concat(hex3, hex2, hex1, hex0);
  return Number(hexa);
}

//
// SQL query
//

function query() {
  var sql = 'select top '+ QUERY_LIMIT + ' cdr.globalCallID_callManagerId, 
       cdr.globalCallID_CallId, 
       cdr.origLegCallIdentifier,
       cdr.destLegIdentifier, 
       cmr.callIdentifier,
       cdr.dateTimeOrigination, 
       cdr.DateTimeConnect,
       cdr.dateTimeDisconnect, 
       cdr.duration, 
       cdr.origIpAddr, 
       cdr.destIpAddr,
       cdr.callingPartyNumber, 
       cdr.originalCalledPartyNumber, 
       cdr.finalCalledPartyNumber, 
       cdr.callingPartyNumberPartition,
       cdr.originalCalledPartyNumberPartition,
       cdr.finalCalledPartyNumberPartition,
       cdr.origDeviceName, 
       cdr.destDeviceName,
       cdr.origCause_value, 
       cdr.destCause_value,
       cmr.numberPacketsSent, 
       cmr.numberOctetsSent,
       cmr.numberPacketsReceived, 
       cmr.numberOctetsReceived,
       cmr.numberPacketsLost, 
       cmr.jitter, 
       cmr.latency
  from dbo.CallDetailRecord cdr left outer join dbo.CallDetailRecordDiagnostic cmr 
    on cmr.globalCallID_callManagerId = cdr.globalCallID_callManagerId
    and cmr.globalCallID_callId = cdr.globalCallID_callId
    and (    cmr.callIdentifier = cdr.origLegCallIdentifier
	     or cmr.callIdentifier = cdr.destLegIdentifier )
 where cdr.dateTimeDisconnect > ' + Persist.LAST_TIME +
    ' order by cdr.dateTimeOrigination,
		  cdr.globalCallID_callManagerId, 
		  cdr.globalCallID_callId, 
		  cdr.origLegCallIdentifier,
		  cdr.destLegIdentifier,
		  cmr.callIdentifier';
  return sql;
}


//
// onRead function (called by CCM.xml design file)
//


function handleCDRCMR(record) {
  if((record.dateTimeDisconnect) || (record.dateTimeDisconnect > 0)) {
    Persist.LAST_TIME = record.dateTimeDisconnect;
//    PV.Logger.logInfo("LAST_TIME update",1,Persist.LAST_TIME);
  }
}


Function.prototype.method = function(name,func) {
  func.name=name;
  this.prototype[name] = func;
  return this;
};



function dump(object) {
  var trace = PV.TraceFile("dump.txt");
  for (var i in object) {
    trace.put("-----------\r",i,"=",object[i],"\r-----------\r");
  }
  trace.close();
}


//
// Configuration Helpers
//


//
//
// The following functions and methods will help to 
// create global variables:
//	- Site's names
//	- Extension Ranges
//	- VoiceMail Ranges
//	- PSTN Gateway IP Addresses
//	- Local Prefix
// Methods defined below are called in the UBA.xml Adaptor configuration file
// Those Variables could have been created manually, with the following syntax:
//
//	SitesByNames = {
//			[ 
//			name:"mySiteName1", 
//			extensionRanges:[[from,to],[from,to], ...], 
//			voiceMailRanges:[[from,to],[from,to], ...], 
//			pstnGatewayIpAddress:[IP1, IP2, ...],
//			localCallPrefixes:["prefix1", "prefix2", ...]
//			],
//	}
//
//


// Constructor
function SiteRange(site,start,stop) {
  this.site=site;
  this.start=start;
  this.stop=stop;
}

SiteRange.method('toString',
	     function() {
		   return this.site.toString()+' ('+this.start+','+this.stop+')';
	     });


// includes(anObjectOrPrimitive)  
// This scan the elements in the Array, comparing each one with the argument.  
// If a match is found, return true, otherwise return false.

SiteRange.method('includes',
	     function(aNumber) {
	       return (aNumber >= this.start && aNumber <= this.stop);
	     });

var SitesByName = {};
var PstnGatewayIpAddresses = new Object();	// Hash Table
var AllExtensionRanges = [];
var AllVoiceMailRanges = [];


function PstnGateway(ipAddress,site) {
  this.address = ipAddress;
  this.site = site;
  return this;
}

PstnGateway.method('toString',
		   function() { 
		     return 'PstnGateway(' + this.address.toString() + ',' + this.site + ')';
		   });

function Site(siteName) {
  var site;
  site = SitesByName[siteName];
  if (site) {
    return site;
  } else {
    this.name = siteName;
    this.extensionRanges = [];
    this.voiceMailRanges = [];
    this.pstnGatewayIpAddresses = new Object();
    this.localCallPrefixes = [];
    SitesByName[siteName] = this;
    return this
  }
}

Site.method('addExtensionRange',
	    function(start,stop) {
	      var range = new SiteRange(this,start,stop);
	      this.extensionRanges.push(range);
	      AllExtensionRanges.push(range);
	    });

// Dump ExtensionRanges Variable
Site.prototype.addExtensionRange.trace=false;


Site.method('addVoiceMailRange',
	    function(start,stop) {
	      var range = new SiteRange(this,start,stop);
	      this.voiceMailRanges.push(range);
	      AllVoiceMailRanges.push(range);
	    });


Site.method('addPstnGatewayIpAddress',
	    function(ipAddressString) {
	      var ipAddress = PV.IpAddress(ipAddressString);
//	      var ipAddress = convertIPAddress2Number(ipAddressString);
//	      this.pstnGatewayIpAddresses[ipAddress.toInteger()]=ipAddress;
//	      PstnGatewayIpAddresses[ipAddress.toInteger()]= new PstnGateway(ipAddress,this);
	      this.pstnGatewayIpAddresses[convertIPAddress2Number(ipAddressString)]=ipAddress;
	      PstnGatewayIpAddresses[convertIPAddress2Number(ipAddressString)]= new PstnGateway(ipAddress,this);
	    });


Site.method('addLocalCallPrefix',
	    function(callPrefix) {
	      this.localCallPrefixes.push(callPrefix);
	    });


Site.method('toString',
	    function () {
	      return 'Site('+this.name+')'});


function getConfigTrace() {
  for (var i in AllExtensionRanges) {
    PV.Logger.logInfo("AllExtensionRanges",1,AllExtensionRanges[i])
  }
  for (var i in AllVoiceMailRanges) {
    PV.Logger.logInfo("AllVoiceMailRanges",1,AllVoiceMailRanges[i])
  }

  for (var i in PstnGatewayIpAddresses) {
    PV.Logger.logInfo("PstnGatewayIpAddresses",1,"PstnGatewayIpAddresses[" + i + "]="+PstnGatewayIpAddresses[i])
  }
}

//
// CONSTRUCTOR FUNCTIONS
//

//
// Extension Object
//
function Extension(numberString) {
  this.string = numberString;
  if (this.string) {
    this.withoutPrefixAndPound = this.removePrefixAndPound(this.string);
    this.number = Number(this.withoutPrefixAndPound);
  } else {
    this.withoutPrefixAndPound = '';
  }
  return this;
}

Extension.method('toString',
		 function() {
		   return this.string;
		 });


Extension.method('removePound',
		 function(string) {
		  // If number ends with "#", this function return the number without the last character
		  if (this.string.indexOf('#') != -1) {
		    // Remove the last digit
		    return this.string.substring(0, this.string.length - 1);
		  } else {
		    return this.string;
		  }
		});

Extension.method('removePrefixAndPound',
		function(string) {
		  return this.removePound(this.removePrefix(string));
		});

Extension.method('removePrefix',
		function(string) {
		  // If string starts with prefix value, this function return the string without the prefix
		  // Caution: this function manages 911 value but remove the prefix in case of 9911
		  if ((string.charAt(0) == OUTBOUNDPREFIX) && ((string.substring(0, 3) != "911") || (string.substring(0, 2) == "00"))) {
		    // Remove the first digit
		    return string.substring(1);
		  } else {
		    return string;
		  }
		});

Extension.method('getNumber',
		 function() {
		   // Test if number is empty
		   if (this.string.length == 0) {
		     return "<UnknownExtension>";
		   } else if (!this.isInternalExtension()) {
		     return "<ExternalNumber>";
		   } else {
		     return "<" + this.string + ">";
		   }
		 });

Extension.method('isPrefix',
		 function() {
		   // If number starts with prefix value, this CDRProto.method('return',function the number without the prefix
		   // Caution: this function manages 911 value but remove the prefix in case of 9911
		   if ((this.string.charAt(0) == OUTBOUNDPREFIX) && 
		       ((this.string.substring(0, 3) != "911") || 
			(this.string.substring(0, 2) == "00"))) {
		     return true;
		   } else {
		     return false;
		   }
		 });

Extension.method('isInternalExtension',
		 function() {
		   // An Internal extension is any private company extension, 
		   // for any site manage by one or several clusters.
			var num = this.number;
			if (this.number) { 
				return AllExtensionRanges.anySatisfy(function(each)  {return each.includes(num) });
			}
			return false;
		 });

Extension.method('isSimpleExtension',
		 function() {
		   // For filter() usage
		   // In order to classify an Extension out of the 5mn UBA aggregation "Object Template" group, 
		   // We need to know whether or not an Extension number is attached to a single user.
		   // Extension subelement that will not be part of the 5mn aggregation are:
		   // - Well known internal extension number

		   return ((this.string.length > 0) && (this.isInternalExtension()));
		 });

Extension.method('isAggregatedExtension',
		 function() {
		   // For filter() usage
		   // In order to classify an Extension in the 5mn UBA aggregation "Object Template" group, 
		   // We need to know whether or not an Extension number is attached to a single user.
		   // Extension subelement that will be part of the 5mn aggregation are:
		   // - ExternalNumber
		   // - UnknownExtension
		   return ((this.string.length == 0) || (!this.isInternalExtension()));
		 });

Extension.method('getSite',
		 function() {
		   // This function is useful to retrieve the site's name where this extension number is installed
		   if ((!this.string) || (this.string.length == 0) || (!this.isInternalExtension())) {
		     // This is not an internal Extension managed by the Cluster, we return an empty string
		     return "";
		   } else {
		     // detect(matchFunction,[ifNoneFunction or non-function]).  
		     // This will enumerate each item in the Array and execute the match function.  
		     // If the match function returns true, the corresponding entry in the Array will be returned.  
		     // If no entry is found, the ifNone function will be executed and its result returned.	
		     // If the ifNone argument isn't a function, the value of  ifNone will simply be returned.  

		     var extensionNumber = this.number;
		     return AllExtensionRanges.detect(function() {return this.includes(extensionNumber)},"").site;
		   }
		 });

Extension.method('getSiteName',
		 function() {
		   // This function is useful to retrieve the site's name where this extension number is installed
		   if ((!this.string) || (this.string.length == 0) || (!this.isInternalExtension())) {
		     // This is not an internal Extension managed by the Cluster, we return an empty string
		     return "";
		   } else {
		     // detect(matchFunction,[ifNoneFunction or non-function]).  
		     // This will enumerate each item in the Array and execute the match function.  
		     // If the match function returns true, the corresponding entry in the Array will be returned.  
		     // If no entry is found, the ifNone function will be executed and its result returned.	
		     // If the ifNone argument isn't a function, the value of  ifNone will simply be returned.  

		     var extensionNumber = this.number;
		     var mySite = AllExtensionRanges.detect(function() {return this.includes(extensionNumber)},"").site;
		     return mySite.name;
		   }
		 });

Extension.method('notSameAs',
		 function(otherExtension) {
		   return this.withoutPrefixAndPound.indexOf(otherExtension.withoutPrefixAndPound) == -1;
		 });

Extension.method('isValid',
		 function() {
		   return this.withoutPrefixAndPound.length > 0;
		 });

Extension.method('length',
		 function() {
		   return this.withoutPrefixAndPound.length;
		 });

Extension.method('prefixMatches',
		 function(aString) {
		   return this.withoutPrefixAndPound.substring(0,aString.length) == aString;
		 });

//
// CAUSE CODE SECTION
//

function defineCauseCodeMetricNames() {
  // one entry for each cause code
var ccmns = {
  0:"NoError",
  1:"UnassignedNumber", 
  2:"NoRouteTransitNetwork",
  3:"NoRouteToDestination",
  5:"MisdialedTrunkPrefix",
  6:"ChannelUnacceptable",
  16:"NormalCallClearing",
  17:"UserBusy",
  18:"NoUserResponding",
  19:"NoAnswerFromUser",
  20:"SubscriberAbsent",
  21:"CallRejected",
  22:"NumberChanged",
  27:"DestinationOutOfOrder",
  28:"InvalidNumberFormat",
  34:"NoChannelAvailable",
  38:"NetworkOutOfOrder",
  41:"TemporaryFailure",
  42:"SwitchCongestion",
  47:"ResourceUnavailable",
  99:"ParameterNotImplemented",
  126:"Transfer"
};

return ccmns;
}


//
// CauseCode Object
//

var CauseCodeMetricNames = defineCauseCodeMetricNames();

function CauseCode(numberString) {
  this.number = Number(numberString);
  return this;
}

CauseCode.method('metricName',
		 function() { 
		   // returns undefined if not found 
		   var name = CauseCodeMetricNames[this.number]; 
		   // undefined tests as false 
		   if (name) { 
		     return FRML_CDR_PATH + "Cause" + name; 
		   } else { 
		     return FRML_CDR_PATH + "CauseOther"; 
		   } 
		 });

CauseCode.method('metricNameForCluster',
		 function() { 
		   // returns undefined if not found 
		   var name = CauseCodeMetricNames[this.number] 
		     // undefined tests as false 
		     if (name) { 
		       return FRML_CLUSTER_PATH + "ClusterCause" + name; 
		     } else { 
		       return FRML_CLUSTER_PATH + "ClusterCauseOther"; 
		     } 
		 });

//
// Record Prototype Methods
//

//
// INTERNAL TOOLS SECTION
//


function defineCDRClass(CDRProto) {

  CDRProto.method = function(name,func) {
    func.name = name;
    this[name] = func;
    return this;
  };



CDRProto.method('areBothInternalExtensions',
		function() {
		  return this.callingPartyNumber.isInternalExtension() && 
		    this.finalCalledPartyNumber.isInternalExtension();
		});

CDRProto.method('isLocalNANP',
		function() {
		  // For US NANP.
		  var length = this.finalCalledPartyNumber.length();
		  return length == 7 || length == 10;
		});


CDRProto.method('isInternationalNANP',
		function() {
		  // For US NANP.
		  // If it starts with  011, return true else return false
		  // this will return either true or undefined 
		  // (undefined will test as false in any if statements or logical expressions)
		  return this.finalCalledPartyNumber.prefixMatches('011');
		});

CDRProto.method('isLongDistanceNANP',
		function() {
		  // For US NANP.
		  return (this.finalCalledPartyNumber.length() == 11) && this.finalCalledPartyNumber.prefixMatches('1');
		});

CDRProto.method('isLocal',
		function() {
		  // return true else return false
		  // this will return either true or undefined 
		  // (undefined will test as false in any if statements or logical expressions)
		  return this.isLocalSiteCall() || this.isLocalNANP();
		});

CDRProto.method('isInternational',
		function() {
		  return (this.finalCalledPartyNumber.prefixMatches('00') || this.isInternationalNANP());
		});

CDRProto.method('isLongDistance',
		function() {		// number is a string
		  // return true else return false
		  // this will return either true or undefined 
		  // (undefined will test as false in any if statements or logical expressions)
		  return ((!this.isInternational()) && 
			  (!this.isLocal()) && 
			  (!this.finalCalledPartyNumber.isInternalExtension()) && 
			  (this.finalCalledPartyNumber.isValid()));
		});

CDRProto.method('isPSTNGateway',
		function(address) {
		  return (address in PstnGatewayIpAddresses);
		});

//
// SUBELEMENT IDENTIFIERS
//


CDRProto.method('isAggregatedExtCallForward',
		function() {
		  // For filter() usage
		  // Extension subelement that will be part of the 5mn aggregation are:
		  // - ExternalNumber
		  // - UnknownExtension
		  // We compare originalCalledPartyNumber and finalCalledPartyNumber.
		  // If values are different for Aggregated Extensions, the call has been forwarded and we return true.
									
		  return (((!this.finalCalledPartyNumber.isValid()) || 
			   (!this.finalCalledPartyNumber.isInternalExtension())) && 
			  (this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber)));
		});

CDRProto.method('isSimpleExtCallForward',function() {
		  // For filter() usage
		  // Extension subelement that will not be part of the 5mn aggregation are:
		  // - Well known extension number
		  // We compare originalCalledPartyNumber and finalCalledPartyNumber.
		  // If values are different for Simple Extensions, the call has been forwarded and we return true.
		  return ((this.finalCalledPartyNumber.isValid()) && 
			  (this.finalCalledPartyNumber.isInternalExtension())) && 
		    (this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber));
		});

CDRProto.method('nullSubelementName',
		function(element) {
		  return element.name + '_<NULL>';
		});
								
CDRProto.method('originatingSubelementName',
		function(element) {
		  return 'Extension_' + element.name + '_' + this.callingPartyNumber.getNumber();
		});

CDRProto.method('destinationSubelementName',
		function(element) { 
		  return 'Extension_' + element.name + '_' + this.originalCalledPartyNumber.getNumber();
		});

CDRProto.method('forwardedSubelementName',
		function(element) {
		  return 'Extension_' + element.name + '_' + this.finalCalledPartyNumber.getNumber();
		});

CDRProto.method('gatewaySubelementName',
		function(element) {
		  return 'Gateway_' + element.name + '_' + this.getGatewayIpAddress().toString();
		});

CDRProto.method('pathSubelementName',
		function(element) {
		  return 'Path_' + element.name + '_' + 
		    this.getPathName (this.callingPartyNumber, 
				      this.finalCalledPartyNumber, 
				      this.origIpAddress, 
				      this.destIpAddress);
		});


//
// UTILITY FUNCTIONS
//
	
CDRProto.method('isSuccessfulCall',
		function() {
		  // If 'dateTimeConnect' value number > 0 the call has been successfully established
		  return	(this.dateTimeConnect > 0);
		});

CDRProto.method('isUnsuccessfulCall',
		function() {
			// An unsuccessful call is first a call where the caller is not a VoiceMail number
			// VM systems send frequent signaling messages to each extension number for MWI management.
			// In this case, the communication is not established, this is just a signalling polling and the call should
			// never be counted as unsuccessful.
			var number = Number(this.callingPartyNumber);
		
			// If 'dateTimeConnect' value number == 0 the call has not been established
		  	// We just check in addition if the callee number is not empty
		  	if (this.finalCalledPartyNumber) {
				return	((this.dateTimeConnect == 0) && (this.finalCalledPartyNumber.length != 0) && (!AllVoiceMailRanges.anySatisfy(function() {return this.includes(number)})));
		  	} else {
				return false;
		  	}
		});

CDRProto.method('isShortCall',
		function() {
		  // A short call is a successful call with duration value == 0 (< 1 second)
		  return ((this.dateTimeConnect != 0) && (this.duration == 0));
		});

CDRProto.method('isDropCall',
		function() {
		  // A Drop Call is first a successfully established call.
		  // Once established, this call is droped if the source or destination cause value is not a normal callClearing
		  if (this.dateTimeConnect > 0) {
		    var source = this.origCause_value.metricName();
		    var dest = this.destCause_value.metricName();
		    if ((source.indexOf("NormalCallClearing") != -1) || (dest.indexOf("NormalCallClearing") != -1)) {
		      return 0;
		    } else {
		      return 1;
		    }
		  } else {
		    return 0;
		  }
		});

CDRProto.method('destinationDuration',
		function() {
		  // This function is usefull only for the originalCalledPartyNumber.
		  // It's important to check first if the call has been forwarded. In this case
		  // we set the callduration value for the intermediate extension (originalCalledPartyNumber)
		  // to 0.
									
		  if (this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber)) {
		    return 0;
		  } else {
		    return this.duration;
		  }
		});

CDRProto.method('hasOriginDetails',
		function() {
		  return ((this.origLegCallIdentifier.indexOf(this.callIdentifier) != -1) && 
			  (this.duration > 0) && (this.numberPacketsSent > 0));
		});

CDRProto.method('hasDestinationDetails',
		function() {
		  return ((this.destLegIdentifier.indexOf(this.callIdentifier) != -1) &&
			  (this.duration > 0) && (this.numberPacketsSent > 0));
		});

CDRProto.method('hasPacketReceived',
		function() {
		  return ((this.duration > 0) && (this.numberPacketsReceived > 0));
		});

CDRProto.method('getPacketsLost',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return negative values for the
		  // PacketsLost metric.
		  // This permit to test if numberPacketsLost is defined
		  if (this.numberPacketsLost > 0) {
			  return this.numberPacketsLost;
		  } else if (this.numberPacketsLost < 0) {
			  return (Math.abs(this.numberPacketsLost));
		  } else {
		  	return 0;
		  }
		});

CDRProto.method('getPacketsSent',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.numberPacketsSent > 0) {
			return this.numberPacketsSent;
		  } else {
			return 0;
		  }
		});

CDRProto.method('getOctetsSent',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.numberOctetsSent > 0) {
			return this.numberOctetsSent;
		  } else {
			return 0;
		  }
		});

CDRProto.method('getPacketsReceived',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.numberPacketsReceived > 0) {
			return this.numberPacketsReceived;
		  } else {
			return 0;
		  }
		});

CDRProto.method('getOctetsReceived',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.numberOctetsReceived > 0) {
			return this.numberOctetsReceived;
		  } else {
			return 0;
		  }
		});

CDRProto.method('getJitter',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.jitter >= 0) {
			return this.jitter;
		  } else {
			return 0;
		  }
		});

CDRProto.method('getLatency',
		function() {
		  // There is a bug in the Cisco CCM Publisher. It appears that many records return NaN values
		  if (this.latency >= 0) {
			return this.latency;
		  } else {
			return 0;
		  }
		});


CDRProto.method('getGatewayEntry',
		function() {
		  var entry;
		  entry = PstnGatewayIpAddresses[this.origIpAddress];
		  if (entry) {
		    return entry}
		  else {
		    return PstnGatewayIpAddresses[this.destIpAddress]}
		});

CDRProto.method('getGatewayIpAddress',
		function() {
		  var entry = this.getGatewayEntry();
		  if (entry) {
		    return entry.address;
		  }
		  return undefined;
		});

CDRProto.method('getSiteFromPSTNGateway',
		function() {
		  // This function is usefull to retrieve the site's name where this PSTN Gateway is installed
		  var entry = this.getGatewayEntry();
		  if (entry) {
		    return entry.site;
		  }
		  return undefined;
		});

CDRProto.method('getSiteNameFromPSTNGateway',
		function() {
		  // This function is usefull to retrieve the site's name where this PSTN Gateway is installed
		  var entry = this.getGatewayEntry();
		  if (entry) {
		    return entry.site.name;
		  }
		  return undefined;
		});

CDRProto.method('isGateway',
		function() {
		  // This function permit to know if at least one of the ip address are a PSTN Gateway
		  // If so, we return "true" in order to create (filter) the PSTN Gateway as a subelement.
									
		  return this.getGatewayEntry();
		});


CDRProto.method('isGatewayCaller',
		function() {
		  // This function permit to know whether or not a PSTN Gateway, 
		  // according to its IP Address is the caller
		  // We compare the value of the sourceIp address with one of the 
		  // address form the PSTNGW global variable

		  var entry = this.getGatewayEntry();


//		  if ((entry) && (entry.address == this.origIpAddress)) {
//		    return 1;
//		  }
		  if (entry) {
		    if (entry.address.toString() == convertInt2IPAddress(this.origIpAddress)) {
		      return 1;
		    }
		  }
		  return 0;
		});									

CDRProto.method('isGatewayCallee',
		function() {
		  // This function permit to know whether or not a PSTN Gateway, 
		  // according to its IP Address is the callee
		  // We compare the value of the destIp address with one of the 
		  // address form the PSTNGW global variable
		  var entry = this.getGatewayEntry();

//		  if ((entry) && (entry.address == this.destIpAddress)) {
//		    return 1;
//		  }
		  if (entry) {
		    if (entry.address.toString() == convertInt2IPAddress(this.destIpAddress)) {
		      return 1;
		    }
		  }
		  return 0;
		});									

CDRProto.method('isPath',
		function() {
		  // This function permit to define a Path subelement
		  // Path definition is when the 2 legs of a call are from different sites
		  // - Internal extension from SiteA to Internal extension from SiteB (SiteA_SiteB)
		  // - Internal extension from SiteA to PSTN Gateway from SiteB (SiteA_SiteB)
		  // - PSTN Gateway from SiteB to Internal extension from SiteA (SiteB_SiteA)
		  // - SiteA to SiteA calls won't create a path
		  return !!this.getPathName();
		});

CDRProto.method('getPathName',
		function() {
		  // This function return the name of a Path
		  // Path definition is when the 2 legs of a call are from different sites
		  // A Path can be a pure IP2IP Path (i.e. Internal call between 2 IP Phones from different sites
		  // or a Path between an Internal IP Phone and a Gateway
		  // - Internal extension from SiteA to Internal extension from SiteB (SiteA_SiteB)
		  // - Internal extension from SiteB to Internal extension from SiteA (SiteB_SiteA)
		  // - Internal extension from SiteA to PSTN Gateway from SiteB (SiteA_GatewayIPAddress)
		  // - PSTN Gateway from SiteB to Internal extension from SiteA (GatewayIPAddress_SiteA)
		  var gwAddress = this.getGatewayIpAddress();
		  var sitecaller = this.callingPartyNumber.getSite();
		  var sitecallee = this.finalCalledPartyNumber.getSite();

		  if (gwAddress == undefined) {
		    // There is no Gateway involved in the call
		    if ((!sitecaller) || (!sitecallee)) {
		      return "";
		    }							
		    if ((sitecaller.name.indexOf(sitecallee.name) != -1) || (sitecaller.name == "") || (sitecallee.name == "")) {
		      // Caller and Callee come from the same Site, and there is no PSTN Gateway.
		      return "";
		    } else {
		      // Caller and Callee come from different sites, return true in order to create the path subelement
		      return "<" +  sitecaller.name + "_" + sitecallee.name + ">";
		    }
		  } else {
		    // This is a call going through a Gateway
		    // The pathname can start with:
		    //	- Caller's site name (caller=ip phone)
		    //	- Gateway IP Address (call comes from the Gateway)
		    if (this.callingPartyNumber.isInternalExtension()  && (sitecaller != "")) {
		      // Caller is an Internal Extension, but can be an IP Phone or a Legacy Phone
		      if (this.isPSTNGateway(this.origIpAddress)) {
			return "<" +  gwAddress.toString() + "_" + sitecallee.name + ">";
		      } else {
			return "<" +  sitecaller.name + "_" + gwAddress.toString() + ">";
		      }
		    } else if (this.finalCalledPartyNumber.isInternalExtension() && (sitecallee != "")) {
		      return "<" +  gwAddress.toString() + "_" + sitecallee.name + ">";
		    }
		  }
		});

CDRProto.method('isLocalSiteCall',
		function() {
		  // This function is useful to manage local call per site and country basis.
		  // Local call policy and numbers are associated with each site's name through the global variables "EXTSITES" and "LOCALCALLS"
		  // LOCALCALLS=["sitename1|prefixNumber1","sitename2|prefixNumber2"]
		  // For each entry in EXTSITES, we compare their site's name with site's name in LOCALCALLS
		  // Then we get the local policy associated with this site
		  // We compare this value with the first N digits of the "number"
		  var finalcallee = this.finalCalledPartyNumber.withoutPrefixAndPound;
		  var siteName = this.callingPartyNumber.getSite();
		  var siteCallPrefixes;
		  if (siteName) {
		    siteCallPrefixes = siteName.localCallPrefixes;
		    for (var i in siteCallPrefixes) {
		      if (siteCallPrefixes[i] == finalcallee.substring(0,siteCallPrefixes[i].length)) {
			return true;
		      }
		    }
		  }
		  return false;
		});


CDRProto.method('outboundExternalCallDuration',
		function() {
		  // An Outbound External Call is a PSTN Call (Local or LD or International)
		  if (this.isLocalCall () || this.isLongDistanceCall () ||this.isInternationalCall ()) {
		    return this.duration;
		  } else {
		    return 0;
		  }
		});


CDRProto.method('isOutboundExternalSuccessfulCall',
		function() {
		  // An Outbound External Call is a PSTN Call (Local or LD or International)
		  if (this.isLocalCall() || this.isLongDistanceCall() || this.isInternationalCall()) {
		    return (this.dateTimeConnect > 0);
		  } else {
		    return 0;
		  }
		});

CDRProto.method('isIncomingSuccessfulCall',
		function() {
		  // Return true if the caller is not an internal extension and the call has been successfuly established
		  return ((this.dateTimeConnect > 0) &&  !this.callingPartyNumber.isInternalExtension());
		});

CDRProto.method('voicemailCallDuration',			function() {
		  if (this.isVoiceMailCall()) {
		    return this.duration;
		  }
		});

CDRProto.method('voicemailPackets',
		function() {
		  // Return the total number of Packets In + Out for a Call to a VoiceMail
		  if (this.isVoiceMailCall()) {
		    var pktsReceived;
		    var pktsSent;
		    if (this.numberPacketsReceived) {
			pktsReceived = this.numberPacketsReceived;
		    } else {
			pktsReceived = 0;
		    }
		    if (this.numberPacketsSent) {
			pktsSent = this.numberPacketsSent;
		    } else {
			pktsSent = 0;
		    }

    	            return (0 + pktsReceived + pktsSent);
		  }
		});

CDRProto.method('meanOpinionScore',
		function() {
		  // Note: THERE IS NO ALGORITHM FOR MEAN OPINION SCORE CALCULATION. 
		  // THIS IS A SUBJECTIVE METRIC FROM THE 
		  // END USER POINT OF VIEW
		  // In voice communications, particularly Internet telephony, 
		  // the mean opinion score (MOS) provides a numerical 
		  // measure of the quality of human speech at the destination end of the circuit. 
		  // The scheme uses subjective tests (opinionated scores) that are mathematically averaged to obtain a 
		  // quantitative indicator of the system performance. 
		  // To determine MOS, a number of listeners rate the quality of test sentences read aloud 
		  // over the communications circuit by male and female speakers. 
		  // A listener gives each sentence a rating as follows: 
		  //	- 1 bad; 
		  //	- 2 poor; 
		  //	- 3 fair; 
		  //	- 4 good;
		  //	- 5 excellent. 
		  // The MOS is the arithmetic mean of all the individual scores, 
		  // and can range from 1 (worst) to 5 (best). 
		  //
		  // This function is a basic implementation of what we call an high level metric about voice quality
									
		  var packetLossAdjustment;
		  var mos = 5;
		  var pktsLost;
		  var pktsReceived;
		  var pktsSent;
		  var Latency;
		  var Jitter;

		  // First thing: Test if any undefined metrics from the CCM DB
		  if (this.numberPacketsReceived) {
			pktsReceived = this.numberPacketsReceived;
		  } else {
			pktsReceived = 0;
		  }
		  if (this.numberPacketsSent) {
			pktsSent = this.numberPacketsSent;
		  } else {
			pktsSent = 0;
		  }
		  if (this.numberPacketsLost) {
			pktsLost = this.numberPacketsLost;
		  } else {
			pktsLost = 0;
		  }
		  if (this.latency) {
			Latency = this.latency;
		  } else {
			Latency = 0;
		  }
		  if (this.jitter) {
			Jitter = this.jitter;
		  } else {
			Jitter = 0;
		  }

		  // We first get the percentage of Packet Lost
		  if ((pktsReceived > 0) || (pktsSent > 0)) {
			  packetLossAdjustment = 100*(pktsLost / (pktsReceived + pktsSent));
		  } else {
			  packetLossAdjustment = 0;
		  }

		  if (packetLossAdjustment < 2) {
		    packetLossAdjustment = 0;
		  } else {
		    packetLossAdjustment = -1;
		  }
									
		  if (Latency < 150) {
		    if (Jitter < 40) {
		      mos = 5;
		    } else {
		      mos = 4;
		    }
		  } else {
		    if (Jitter < 40) {
		      mos = 4;
		    } else {
		      mos = 3;
		    }
		  }
		  return (mos + packetLossAdjustment);
		});


CDRProto.method('tollbypassCallDuration',
		function() {
		  if (this.isTollBypassCall () > 0 ) {

		  }
		});


//
// CALL CLASSIFICATION SECTION
//

CDRProto.method('isInternalCall',
		function() {
		  // An Internal Call is define as a call where caller + finalcallee are Internal Extensions
									
		  // Test if number is an internal extension or external
		  return (this.callingPartyNumber.isInternalExtension() && this.finalCalledPartyNumber.isInternalExtension());
		});

CDRProto.method('isOnNetCall',
		function() {
		  // An OnNet Call is define as a call where caller is an internal extension
		  // and the gateway ip address is not in the global PSTNGW list.
		  // An OnNet call is a call between 2 IP endpoints which doesn't go through PSTN
		  return (this.callingPartyNumber.isInternalExtension() && !this.isPSTNGateway(this.destIpAddress));
		});

CDRProto.method('isOffNetCall',
		function() {
		  // An OffNet Call is define as a call where one of the ip addresses is in the global PSTNGW list.
		  // An OffNet call is a call which does go through the PSTN or any Legacy PBX
		  return this.isPSTNGateway(this.origIpAddress) || this.isPSTNGateway(this.destIpAddress);
		});

CDRProto.method('isIncomingCall',
		function() {
		  // An Incoming Call is define as a call where caller is not an internal extension
		  // Test if number is an internal extension or not
		  return (!this.callingPartyNumber.isInternalExtension());
		});

CDRProto.method('isLocalCall',
		function() {
		  // A Local Call is first an OffNet call, going through PSTN Gateway
		  // Calling number has to be an Internal Extension
		  // Called number has to match with isLocal() function
		  // Gateway ip address has to match with on of the address in the global PSTNGW list.
									
		  // Test if number is an internal extension or external
		  return (this.callingPartyNumber.isInternalExtension() && 
			  this.isPSTNGateway(this.destIpAddress) && this.isLocal());
		});

CDRProto.method('isInternationalCall',
		function() {
		  // An International Call is first an OffNet call, going through PSTN Gateway
		  // Calling number has to be an Internal Extension
		  // Called number has to match with isInternational() function
		  // Gateway ip address has to match with on of the address in the global PSTNGW list.
									
		  // Test if number is an internal extension or external
		  return (this.callingPartyNumber.isInternalExtension() && this.isInternational());
		});

CDRProto.method('isLongDistanceCall',
		function() {
		  // A LongDistance Call is first an OffNet call, going through PSTN Gateway
		  // Calling number has to be an Internal Extension
		  // Called number has to match with isLongDistance() function
		  // Gateway ip address has to match with on of the address in the global PSTNGW list.
									
		  // Test if number is an internal extension or external
		  return (this.callingPartyNumber.isInternalExtension() && 
			  this.isPSTNGateway(this.destIpAddress) && 
			  this.isLongDistance());
		});

CDRProto.method('isVoiceMailCall',
		function() {
		  // A VoiceMail Call is define as a call where the finalcallee is equal to 
		  // one of the value of the global variable VOICEMAILEXTENSION
		  // VOICEMAILEXTENSION = ["sitename|extension#-extension#"]
		  // This is a range of extensions.
		  // Remove prefix if any, remove # if any
		  var finalcallee = this.finalCalledPartyNumber.withoutPrefixAndPound;
									
		  if (!finalcallee) {
		    // This is not a VoiceMail Number
		    return false;
		  } else {
		    var number = Number(finalcallee);
//		    return AllVoiceMailRanges.anySatisfy(function() {return this.includes(number)});		BUG HERE
                    for (var i in AllVoiceMailRanges) {
                      if ((number >= AllVoiceMailRanges[i].start) && (number <= AllVoiceMailRanges[i].stop)) {
                        return 1;
                      } else {
                        return 0;
                      }
                    }
		  }
		});

CDRProto.method('isTollBypassCall',
		function() {
		  // A TollBypass Call is define as a call where caller's extension and the PSTN Gateway where the
		  // call goes through are from different locations (different sites)
		  // Caller has to be an internal extension
		  if (this.callingPartyNumber.isInternalExtension()) {
		    var sitegw = this.getSiteFromPSTNGateway ();
		    var siteext = this.callingPartyNumber.getSite();
		    if (!sitegw || !siteext) {
			return 0;
		    } else {
		        if (sitegw.name.indexOf(siteext.name) != -1) {
		          return 0;
		        } else {
		          return 1;
			}
		    }
		  } else {
		    return 0;
		  }
		});

CDRProto.method('isIntrasiteIPCall',
		function() {
		  // An Intrasite Call is define as a call between 2 IP Phones from the same site.
		  // In this case IP addresses for the 2 endpoints are not in the PSTNGW list
		  if (this.areBothInternalExtensions()) {
		    var sitegw = this.getSiteFromPSTNGateway();
		    var sitecalling = this.callingPartyNumber.getSite();
		    var sitefinalcallee = this.finalCalledPartyNumber.getSite();
		    if (!sitegw && (sitecalling.name.indexOf(sitefinalcallee.name) != -1)) {
		      return 1;
		    } else {
		      return 0;
		    }
		  } else {
		    return 0;
		  }
		});

CDRProto.method('isIntersiteIPCall',
		function() {
		  // An Intersite Call is define as a call between 2 IP Phones from the different sites.
		  // In this case IP addresses for the 2 endpoints are not in the PSTNGW list
		  if (this.areBothInternalExtensions()) {
		    var sitecalling = this.callingPartyNumber.getSite();
		    var sitefinalcallee = this.finalCalledPartyNumber.getSite();
		    var sitegw = this.getSiteFromPSTNGateway();
		    if ((sitecalling != "") && (sitefinalcallee != "")) {
			    if (!sitegw && (sitecalling.name.indexOf(sitefinalcallee.name) == -1)) {
			      return 1;
			    } else {
			      return 0;
			    }
		    } else {
			return 0;
		    }
		  } else {
		    return 0;
		  }
		});

CDRProto.method('isIntrasiteLegacyPBXCall',
		function() {
		  // An Intrasite LegaciyPBX Call is define as a call between 1 IP Phone 
		  // and 1 Analog phone connected to a legacy PBX.
		  // Those 2 phones are connected to the same site.
		  // In this case one of the IP address is in the PSTNGW list
		  if (this.areBothInternalExtensions()) {
		    var sitegw = this.getSiteFromPSTNGateway();
		    var sitecalling = this.callingPartyNumber.getSite();
		    var sitefinalcallee = this.finalCalledPartyNumber.getSite();
		    if (!sitegw && (sitecalling.name.indexOf(sitefinalcallee.name) != -1)) {
		      return 1;
		    } else {
		      return 0;
		    }
		  } else {
		    return 0;
		  }
		});

CDRProto.method('isIntersiteLegacyPBXCall',
		function() {
		  // An Intersite LegacyPBX Call is define as a call between 1 IP Phone
		  // and 1 Analog phone connected to a legacy PBX.
		  // Those 2 phones are connected to different sites.
		  // In this case one of the IP address is in the PSTNGW list
		  if (this.areBothInternalExtensions()) {
		    var sitegw = this.getSiteFromPSTNGateway();
		    var sitecalling = this.callingPartyNumber.getSite();
		    var sitefinalcallee = this.finalCalledPartyNumber.getSite();
		    if ((sitegw != "") && (sitecalling.name.indexOf(sitefinalcallee.name) == -1)) {
		      return 1;
		    } else {
		      return 0;
		    }
		  } else {
		    return 0;
		  }
		});

//
// CALL FORWARD SECTION
//

CDRProto.method('isCallForwardToVoiceMail',
		function() {
		  // Does the call has been forwarded to the VoiceMail extension number
		  // originalcallee and final callee has to be different
		  // and
		  // finalcallee has to be equal to VOICEMAILEXTENSION value

		  return ((this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber)) &&
			  (this.isVoiceMailCall()));
		});

CDRProto.method('isCallForwardToExtension',
		function() {
		  // Does the call has been forwarded to an internal extension
		  // originalcallee and final callee has to be different
		  // and
		  // finalcallee has to be an internal extension number
		  // Remove prefix if any, remove # if any
									
		  return (this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber) && 
			  (this.finalCalledPartyNumber.isInternalExtension()));
		});

CDRProto.method('isCallForwardToPSTN',
		function() {
		  // Has the call been forwarded to an external number?
		  // finalcallee is not empty
		  // originalcallee and final callee are different
		  // finalcallee is not an internal extension
		  // finalcalle is not a voicemail extension
		  if (this.callingPartyNumber && this.finalCalledPartyNumber) {
		    return (this.finalCalledPartyNumber.isValid()
			    && (this.originalCalledPartyNumber.notSameAs(this.finalCalledPartyNumber))
			    && (!this.finalCalledPartyNumber.isInternalExtension()) 
			    && (!this.isVoiceMailCall()));
		  } else {
		    return 0;
		  }
		});

}

